"""victoria_email.core

This is the core functionality of the email plugin.

Code in this package is converted one-to-one from numerous disparate scripts,
so may not fit the formatting/style of other code.

You may have previously known this functionality as 'MailToil'.

Author:
    Sam Gibson <sgibson@glasswallsolutions.com>
"""